@@ -717,8 +717,7 @@ func TestLoadWithFormat(t *testing.T) {
 		"ZOT": "QIX",
 	}
 
-	custom := func(r io.Reader, _ string, lookup func(key string) (string, bool)) (map[string]string, error) {
-		vars := map[string]string{}
+	custom := func(r io.Reader, _ string, vars map[string]string, lookup func(key string) (string, bool)) error {
 		scanner := bufio.NewScanner(r)
 		for scanner.Scan() {
 			key, value, found := strings.Cut(scanner.Text(), ":")
@@ -730,14 +729,46 @@ func TestLoadWithFormat(t *testing.T) {
 			}
 			vars[key] = value
 		}
-		return vars, nil
+		return nil
 	}
 
 	RegisterFormat("custom", custom)
 
 	f, err := os.Open(envFileName)
 	assert.NilError(t, err)
-	env, err := ParseWithFormat(f, envFileName, nil, "custom")
+	env := map[string]string{}
+	err = ParseWithFormat(f, envFileName, env, nil, "custom")
 	assert.NilError(t, err)
 	assert.DeepEqual(t, expectedValues, env)
 }
+
+func TestMultipleFiles(t *testing.T) {
+	base := filepath.Join(t.TempDir(), "base.env")
+	err := os.WriteFile(base, []byte(`
+ENV_HOSTNAME=localhost
+ENV_MY_URL="http://${ENV_HOSTNAME}"
+`), 0o600)
+	assert.NilError(t, err)
+
+	override := filepath.Join(t.TempDir(), "override.env")
+	err = os.WriteFile(override, []byte(`
+ENV_HOSTNAME=dev.my-company.com
+ENV_MY_URL="http://${ENV_HOSTNAME}"
+`), 0o600)
+	assert.NilError(t, err)
+
+	env, err := GetEnvFromFile(nil, []string{base, override})
+	assert.NilError(t, err)
+	assert.DeepEqual(t, env, map[string]string{
+		"ENV_HOSTNAME": "dev.my-company.com",
+		"ENV_MY_URL":   "http://dev.my-company.com",
+	})
+
+	osEnv := map[string]string{"ENV_HOSTNAME": "host.local"}
+	env, err = GetEnvFromFile(osEnv, []string{base, override})
+	assert.NilError(t, err)
+	assert.DeepEqual(t, env, map[string]string{
+		"ENV_HOSTNAME": "dev.my-company.com",
+		"ENV_MY_URL":   "http://host.local",
+	})
+}
