@@ -3050,7 +3050,7 @@ services:
         # sync static content
         - path: ./webapp/html
           action: sync
-          x-initialSync: true
+          initial_sync: true
           target: /var/www
           ignore:
             - node_modules/
@@ -3082,13 +3082,11 @@ services:
 	assert.DeepEqual(t, *frontend.Develop, types.DevelopConfig{
 		Watch: []types.Trigger{
 			{
-				Path:   "./webapp/html",
-				Action: types.WatchActionSync,
-				Target: "/var/www",
-				Ignore: []string{"node_modules/"},
-				Extensions: types.Extensions{
-					"x-initialSync": true,
-				},
+				Path:        "./webapp/html",
+				Action:      types.WatchActionSync,
+				Target:      "/var/www",
+				Ignore:      []string{"node_modules/"},
+				InitialSync: true,
 			},
 		},
 	})
@@ -3919,3 +3917,65 @@ services:
 	assert.Equal(t, build.Provenance, "mode=max")
 	assert.Equal(t, build.SBOM, "true")
 }
+
+func TestOverrideMiddle(t *testing.T) {
+	pwd := t.TempDir()
+	base := filepath.Join(pwd, "base.yaml")
+	err := os.WriteFile(base, []byte(`
+services:
+  base:
+    volumes:
+      - /foo:/foo
+`), 0o700)
+	assert.NilError(t, err)
+
+	override := filepath.Join(pwd, "override.yaml")
+	err = os.WriteFile(override, []byte(`
+services:
+  override:
+    extends:
+      file: ./base.yaml
+      service: base
+    volumes: !override
+      -  /bar:/bar
+`), 0o700)
+	assert.NilError(t, err)
+
+	compose := filepath.Join(pwd, "compose.yaml")
+	err = os.WriteFile(compose, []byte(`
+name: test
+services:
+  test:
+    image: test
+    extends:
+      file: ./override.yaml
+      service: override
+    volumes:
+      - /zot:/zot
+`), 0o700)
+	assert.NilError(t, err)
+
+	project, err := LoadWithContext(context.TODO(), types.ConfigDetails{
+		WorkingDir: pwd,
+		ConfigFiles: []types.ConfigFile{
+			{Filename: compose},
+		},
+	})
+	assert.NilError(t, err)
+	test := project.Services["test"]
+	assert.Equal(t, len(test.Volumes), 2)
+	assert.DeepEqual(t, test.Volumes, []types.ServiceVolumeConfig{
+		{
+			Type:   "bind",
+			Source: "/bar",
+			Target: "/bar",
+			Bind:   &types.ServiceVolumeBind{CreateHostPath: true},
+		},
+		{
+			Type:   "bind",
+			Source: "/zot",
+			Target: "/zot",
+			Bind:   &types.ServiceVolumeBind{CreateHostPath: true},
+		},
+	})
+}
