@@ -44,8 +44,8 @@ import (
 )
 
 var provenanceTests = integration.TestFuncs(
-	testProvenanceAttestation,
-	testGitProvenanceAttestation,
+	testGitProvenanceAttestationSHA1,
+	testGitProvenanceAttestationSHA256,
 	testMultiPlatformProvenance,
 	testClientFrontendProvenance,
 	testClientLLBProvenance,
@@ -71,6 +71,8 @@ func testProvenanceAttestation(t *testing.T, sb integration.Sandbox) {
 	workers.CheckFeatureCompat(t, sb, workers.FeatureDirectPush, workers.FeatureProvenance)
 	ctx := sb.Context()
 
+	isDockerd := strings.HasPrefix(sb.Name(), "dockerd")
+
 	c, err := client.New(ctx, sb.Address())
 	require.NoError(t, err)
 	defer c.Close()
@@ -179,6 +181,14 @@ RUN echo ok> /foo
 				_, isClient := f.(*clientFrontend)
 				_, isGateway := f.(*gatewayFrontend)
 
+				expCustom := provenancetypes.ProvenanceCustomEnv{
+					"foo":     "bar",
+					"numbers": []any{1.0, 2.0, 3.0},
+				}
+				if isDockerd {
+					expCustom = nil
+				}
+
 				if slsaVersion == "v1" {
 					type stmtT struct {
 						Predicate provenancetypes.ProvenancePredicateSLSA1 `json:"predicate"`
@@ -192,6 +202,8 @@ RUN echo ok> /foo
 
 					require.Equal(t, "", pred.BuildDefinition.ExternalParameters.ConfigSource.URI)
 
+					require.Equal(t, expCustom, pred.BuildDefinition.InternalParameters.ProvenanceCustomEnv)
+
 					args := pred.BuildDefinition.ExternalParameters.Request.Args
 					if isClient {
 						require.Equal(t, "", pred.BuildDefinition.ExternalParameters.Request.Frontend)
@@ -293,6 +305,8 @@ RUN echo ok> /foo
 
 					require.Equal(t, "", pred.Invocation.ConfigSource.URI)
 
+					require.Equal(t, expCustom, pred.Invocation.Environment.ProvenanceCustomEnv)
+
 					args := pred.Invocation.Parameters.Args
 					if isClient {
 						require.Equal(t, "", pred.Invocation.Parameters.Frontend)
@@ -389,7 +403,15 @@ RUN echo ok> /foo
 	}
 }
 
-func testGitProvenanceAttestation(t *testing.T, sb integration.Sandbox) {
+func testGitProvenanceAttestationSHA1(t *testing.T, sb integration.Sandbox) {
+	testGitProvenanceAttestation(t, sb, "sha1")
+}
+
+func testGitProvenanceAttestationSHA256(t *testing.T, sb integration.Sandbox) {
+	testGitProvenanceAttestation(t, sb, "sha256")
+}
+
+func testGitProvenanceAttestation(t *testing.T, sb integration.Sandbox, format string) {
 	integration.SkipOnPlatform(t, "windows")
 	workers.CheckFeatureCompat(t, sb, workers.FeatureDirectPush, workers.FeatureProvenance)
 	ctx := sb.Context()
@@ -423,8 +445,12 @@ COPY myapp.Dockerfile /
 				fstest.CreateFile("myapp.Dockerfile", dockerfile, 0600),
 			)
 
+			initOptions := ""
+			if format == "sha256" {
+				initOptions = " --object-format=sha256"
+			}
 			err = runShell(dir.Name,
-				"git init",
+				"git init"+initOptions,
 				"git config --local user.email test",
 				"git config --local user.name test",
 				"git add myapp.Dockerfile",
@@ -519,7 +545,7 @@ COPY myapp.Dockerfile /
 					require.NotEmpty(t, pred.BuildDefinition.ResolvedDependencies[1].Digest["sha256"])
 
 					require.Equal(t, expectedURL+"/.git#v1", pred.BuildDefinition.ResolvedDependencies[2].URI)
-					require.Equal(t, strings.TrimSpace(string(expectedGitSHA)), pred.BuildDefinition.ResolvedDependencies[2].Digest["sha1"])
+					require.Equal(t, strings.TrimSpace(string(expectedGitSHA)), pred.BuildDefinition.ResolvedDependencies[2].Digest[format])
 				} else {
 					require.Equal(t, 2, len(pred.BuildDefinition.ResolvedDependencies), "%+v", pred.BuildDefinition.ResolvedDependencies)
 
@@ -527,7 +553,7 @@ COPY myapp.Dockerfile /
 					require.NotEmpty(t, pred.BuildDefinition.ResolvedDependencies[0].Digest["sha256"])
 
 					require.Equal(t, expectedURL+"/.git#v1", pred.BuildDefinition.ResolvedDependencies[1].URI)
-					require.Equal(t, strings.TrimSpace(string(expectedGitSHA)), pred.BuildDefinition.ResolvedDependencies[1].Digest["sha1"])
+					require.Equal(t, strings.TrimSpace(string(expectedGitSHA)), pred.BuildDefinition.ResolvedDependencies[1].Digest[format])
 				}
 
 				require.Equal(t, 0, len(pred.BuildDefinition.ExternalParameters.Request.Locals))
@@ -574,7 +600,7 @@ COPY myapp.Dockerfile /
 					require.NotEmpty(t, pred.Materials[1].Digest["sha256"])
 
 					require.Equal(t, expectedURL+"/.git#v1", pred.Materials[2].URI)
-					require.Equal(t, strings.TrimSpace(string(expectedGitSHA)), pred.Materials[2].Digest["sha1"])
+					require.Equal(t, strings.TrimSpace(string(expectedGitSHA)), pred.Materials[2].Digest[format])
 				} else {
 					require.Equal(t, 2, len(pred.Materials), "%+v", pred.Materials)
 
@@ -582,7 +608,7 @@ COPY myapp.Dockerfile /
 					require.NotEmpty(t, pred.Materials[0].Digest["sha256"])
 
 					require.Equal(t, expectedURL+"/.git#v1", pred.Materials[1].URI)
-					require.Equal(t, strings.TrimSpace(string(expectedGitSHA)), pred.Materials[1].Digest["sha1"])
+					require.Equal(t, strings.TrimSpace(string(expectedGitSHA)), pred.Materials[1].Digest[format])
 				}
 
 				require.Equal(t, 0, len(pred.Invocation.Parameters.Locals))
@@ -1994,3 +2020,51 @@ COPY --from=base /out /
 		require.NoError(t, json.Unmarshal(dt, &pred))
 	}
 }
+
+type provenanceEnvSimple struct{}
+
+func (*provenanceEnvSimple) UpdateConfigFile(in string) (string, func() error) {
+	dir, err := os.MkdirTemp("", "provenanceenv")
+	if err != nil {
+		panic(err)
+	}
+	dt, err := json.Marshal(map[string]any{
+		"foo": "bar",
+	})
+	if err != nil {
+		panic(err)
+	}
+	if err := os.WriteFile(filepath.Join(dir, "foo.json"), dt, 0600); err != nil {
+		panic(err)
+	}
+	dt, err = json.Marshal(map[string]any{
+		"numbers": []int{1, 2, 3},
+	})
+	if err != nil {
+		panic(err)
+	}
+	if err := os.WriteFile(filepath.Join(dir, "numbers.json"), dt, 0600); err != nil {
+		panic(err)
+	}
+
+	// make all paths readable for the rootless user
+	if err := os.Chmod(dir, 0755); err != nil {
+		panic(err)
+	}
+	if err := os.Chmod(filepath.Join(dir, "foo.json"), 0644); err != nil {
+		panic(err)
+	}
+	if err := os.Chmod(filepath.Join(dir, "numbers.json"), 0644); err != nil {
+		panic(err)
+	}
+
+	in = in + fmt.Sprintf("\n\nprovenanceEnvDir = %q\n", dir)
+
+	return in, func() error {
+		return os.RemoveAll(dir)
+	}
+}
+
+var (
+	provenanceEnvSimpleConfig integration.ConfigUpdater = &provenanceEnvSimple{}
+)
